Module bayesvalidrox.surrogate_models.exploration
=================================================

Classes
-------

`Exploration(MetaModel, n_candidate, mc_criterion='mc-intersite-proj-th')`
:   Created based on the Surrogate Modeling Toolbox (SUMO) [1].
    
    [1] Gorissen, D., Couckuyt, I., Demeester, P., Dhaene, T. and Crombecq, K.,
        2010. A surrogate modeling and adaptive sampling toolbox for computer
        based design. Journal of machine learning research.-Cambridge, Mass.,
        11, pp.2051-2055. sumo@sumo.intec.ugent.be - http://sumo.intec.ugent.be
    
    Attributes
    ----------
    MetaModel : obj
        MetaModel object.
    n_candidate : int
        Number of candidate samples.
    mc_criterion : str
        Selection crieterion. The default is `'mc-intersite-proj-th'`. Another
        option is `'mc-intersite-proj'`.
    w : int
        Number of random points in the domain for each sample of the
        training set.

    ### Methods

    `get_exploration_samples(self)`
    :   This function generates candidates to be selected as new design and
        their associated exploration scores.
        
        Returns
        -------
        all_candidates : array of shape (n_candidate, n_params)
            A list of samples.
        exploration_scores: arrays of shape (n_candidate)
            Exploration scores.

    `get_vornoi_samples(self)`
    :   This function generates samples based on voronoi cells and their
        corresponding scores
        
        Returns
        -------
        new_samples : array of shape (n_candidate, n_params)
            A list of samples.
        exploration_scores: arrays of shape (n_candidate)
            Exploration scores.

    `get_mc_samples(self, all_candidates=None)`
    :   This function generates random samples based on Global Monte Carlo
        methods and their corresponding scores, based on [1].
        
        [1] Crombecq, K., Laermans, E. and Dhaene, T., 2011. Efficient
            space-filling and non-collapsing sequential design strategies for
            simulation-based modeling. European Journal of Operational Research
            , 214(3), pp.683-696.
            DOI: https://doi.org/10.1016/j.ejor.2011.05.032
        
        Implemented methods to compute scores:
            1) mc-intersite-proj
            2) mc-intersite-proj-th
        
        Arguments
        ---------
        all_candidates : array, optional
            Samples to compute the scores for. The default is `None`. In this
            case, samples will be generated by defined model input marginals.
        
        Returns
        -------
        new_samples : array of shape (n_candidate, n_params)
            A list of samples.
        exploration_scores: arrays of shape (n_candidate)
            Exploration scores.

    `approximate_voronoi(self, w, samples)`
    :   An approximate (monte carlo) version of Matlab's voronoi command.
        
        Arguments
        ---------
        samples : array
            Old experimental design to be used as center points for voronoi
            cells.
        
        Returns
        -------
        areas : array
            An approximation of the voronoi cells' areas.
        all_candidates: list of arrays
            A list of samples in each voronoi cell.